package entity;


/** Ein Intervall ist begrenzt von zwei (Zeit)-Punkten,
 * von denen der erste vor dem zweiten liegen muss, damit 
 * es sich um ein korrektes Intervall handelt. Die linke
 * und die rechte Intervallgrenze gehoeren jeweils zum
 * Intervall dazu. Zwei Intervalle koennen in verschiedenen 
 * Verhaeltnissen zueinander liegen und gemeinsame Punkte 
 * umfassen.
 * @param <T> Typ zur Bescheibung der Intervallgrenzen
 */
public interface Intervall<T> {
    /** Ueberueft, ob es sich um ein korrektes Intervall
     * handelt.
     * @return liegt der erste Punkt vor dem zweiten Punkt?
     */
    public boolean istIntervall();
    
    /** Ueberprueft, ob die Intervalle getrennte Bereiche
     * beschreiben, dabei soll dies auch der Fall ein, wenn 
     * Anfangs- und Endpunkt beziehungsweise andersherum 
     * der Intervallgrenzen uebereinstimmen.
     * Sollte es sich bei mindestens einem der Intervalle um
     * kein echtes Intervall handeln, ist das Ergebnis false.
     * @param intervall zweites, zu vergleichendes Intervall
     * @return sind beide Intervalle getrennt voneinander?
     */
    public boolean istDisjunktMit(Intervall<T> intervall);
    
    /** Ueberprueft, ob dieses Intervall (this), in dem
     * uebergebenen Intervall liegt, dabei gilt dies auch, 
     * wenn die Intervalle gemeinsame Grenzen haben.
     * Sollte es sich bei mindestens einem der Intervalle um
     * kein echtes Intervall handeln, ist das Ergebnis false.
     * @param intervall zweites, zu vergleichendes Intervall
     * @return liegt das Intervall in dem uebergebenen Intervall?
     */
    public boolean istEnthaltenIn(Intervall<T> intervall);
    
    /** Liefert das Intervall, in dem sich beide Intervalle
     * ueberschneiden, sollte dies nicht der Fall oder es 
     * nur ein Punkt sein, ist das Ergebnis null.
     * Sollte es sich bei mindestens einem der Intervalle um
     * kein echtes Intervall handeln, ist das Ergebnis null.
     * @param intervall zweites, zu schneidendes Intervall
     * @return gemeinsames Teilintervall beider Intervalle,
     *         wenn vorhanden, ansonsten null
     */
    public Intervall<T> schneidenMit(Intervall<T> intervall);
    
    /** Gibt den Startpunkt des Intervalls.
     * @return Startpunkt des Intervalls
     */
    public T getStart();
 
    /** Gibt den Endpunkt des Intervalls.
     * @return Endpunkt des Intervalls
     */    
    public T getEnde();
    
    /** Setzt den Startpunkt.
     * @param punkt neuer Startpunkt
     */
    public void setStart(T punkt);
    
    /** Setzt den Endpunkt.
     * @param punkt neuer Endpunkt
     */
    public void setEnde(T punkt);    
}
